@@ -20,7 +20,7 @@ module HOpenPGP.Tools.TKUtils (
     processTK
 ) where
 
-import Codec.Encryption.OpenPGP.Fingerprint (eightOctetKeyID)
+import Codec.Encryption.OpenPGP.Fingerprint (eightOctetKeyID, fingerprint)
 import Codec.Encryption.OpenPGP.Signatures (verifyTKWith, verifySigWith, verifyAgainstKeys)
 import Codec.Encryption.OpenPGP.Types
 import Control.Error.Util (hush)
@@ -46,17 +46,26 @@ processTK mpt key = verifyTKWith (verifySigWith (verifyAgainstKeys [key])) (fmap
         }
         newest = take 1 . sortOn (Down . take 1 . sigcts) -- FIXME: this is terrible
         sigcts (SigV4 _ _ _ xs _ _ _) = map (\(SigSubPacket _ (SigCreationTime x)) -> x) $ filter isCT xs
-        alleged = filter (\x -> ((==) <$> sigissuer x <*> eoki (key^.tkKey._1)) == Just True)
+        pkp = key^.tkKey._1
+        alleged = filter (\x -> assI x || assIFP x)
         isCT (SigSubPacket _ (SigCreationTime _)) = True
         isCT _ = False
         sigissuer (SigVOther 2 _) = Nothing
         sigissuer SigV3{} = Nothing
         sigissuer (SigV4 _ _ _ ys xs _ _) = listToMaybe . mapMaybe (getIssuer . _sspPayload) $ (ys++xs) -- FIXME: what should this be if there are multiple matches?
         sigissuer (SigVOther _ _) = error "We're in the future." -- FIXME
-        eoki pkp
+        sigissuerfp (SigV4 _ _ _ ys xs _ _) = listToMaybe . mapMaybe (getIssuerFP . _sspPayload) $ (ys++xs) -- FIXME: what should this be if there are multiple matches?
+        sigissuerfp _ = Nothing
+        eoki
             | pkp^.keyVersion == V4 = hush . eightOctetKeyID $ pkp
             | pkp^.keyVersion == DeprecatedV3 && elem (pkp^.pkalgo) [RSA,DeprecatedRSASignOnly] = hush . eightOctetKeyID $ pkp
             | otherwise = Nothing
+        fp
+            | pkp^.keyVersion == V4 = Just . fingerprint $ pkp
+            | otherwise = Nothing
         getIssuer (Issuer i) = Just i
         getIssuer _ = Nothing
-
+        getIssuerFP (IssuerFingerprint 4 i) = Just i
+        getIssuerFP _ = Nothing
+        assI x = ((==) <$> sigissuer x <*> eoki) == Just True
+        assIFP x = ((==) <$> sigissuerfp x <*> fp) == Just True
